'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports['default'] = createFormReducer;

var _actionTypes = require('./actionTypes');

/**
 * Creates a state structure like:
 * {
 *   data: {
 *     field1: 'value1',
 *     field2: 'value2'
 *   },
 *   visited: {
 *     field1: true,
 *     field2: false
 *   }
 * }
 *
 * A field has been "visited" if its value has been updated since the creation of the reducer.
 *
 * @param name the name of the "state slice" where the data is stored
 * @param fields an array of field names, used when showing all values
 * @param initialData initial data to populate the state with
 * @returns {Function} a form reducer
 */

function createFormReducer(name, fields) {
  var initialData = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  var initialState = { data: initialData, visited: {} };
  return function () {
    var state = arguments.length <= 0 || arguments[0] === undefined ? initialState : arguments[0];

    var _extends2, _extends3;

    var action = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

    if (action.form !== name) {
      return state;
    }
    switch (action.type) {
      case _actionTypes.CHANGE:
        var data = _extends({}, state.data, (_extends2 = {}, _extends2[action.field] = action.value, _extends2));
        return _extends({}, state, {
          data: data,
          visited: _extends({}, state.visited, (_extends3 = {}, _extends3[action.field] = true, _extends3))
        });
      case _actionTypes.SHOW_ALL:
        var visited = {};
        fields.forEach(function (key) {
          return visited[key] = true;
        }); // mark all as visited
        return _extends({}, state, {
          visited: visited
        });
      case _actionTypes.RESET:
        return initialState;
      default:
        return state;
    }
  };
}

module.exports = exports['default'];